#include "precompile.h"

#include <google/protobuf/message.h>
#include <a8/httpsession.h>
#include <a8/tcplistener.h>

#include "framework/cpp/netmsghandler.h"

#include "app.h"
#include "GCListener.h"
#include "jsondatamgr.h"
#include "handlermgr.h"

class GCClientSession: public a8::HttpSession
{
public:

    virtual void OnRawHttpGet(const std::string& url, const std::string& querystr,
                              std::string& response) override
    {
        App::Instance()->AddIMMsg(IM_ExecGM,
                                  a8::XParams()
                                  .SetSender(socket_handle)
                                  .SetParam1(url)
                                  .SetParam2(querystr)
                                  .SetParam3(saddr));
    }

    virtual void OnDisConnect() override
    {
        App::Instance()->AddIMMsg(IM_ClientSocketDisconnect,
                                  a8::XParams()
                                  .SetSender(socket_handle)
                                  .SetParam1(1));
    }

};

static void CreateGameClientSocket(a8::TcpSession **p)
{
    *p = new GCClientSession();
}

static void GSListeneron_error(a8::TcpListener*, int type, int errorid)
{
    a8::UdpLog::Instance()->Debug("GCListeneron_error %d %d", {type, errorid});
}

void GCListener::Init()
{
    tcp_listener_ = new a8::TcpListener();
    tcp_listener_->on_create_client_socket = CreateGameClientSocket;
    tcp_listener_->on_error = GSListeneron_error;

    tcp_listener_->bind_address = "0.0.0.0";
    tcp_listener_->bind_port = JsonDataMgr::Instance()->GetConf()->At("listen_port")->AsXValue();
    tcp_listener_->Open();
}

void GCListener::UnInit()
{
    delete tcp_listener_;
    tcp_listener_ = nullptr;
}

void GCListener::SendText(unsigned short sockhandle, const std::string& text)
{
    tcp_listener_->SendClientMsg(sockhandle, text.data(), text.size());
}

void GCListener::ForceCloseClient(unsigned short sockhandle)
{
    tcp_listener_->ForceCloseClient(sockhandle);
}

void GCListener::MarkClient(unsigned short sockhandle, bool is_active)
{
    tcp_listener_->MarkClient(sockhandle, is_active);
}
